@@ -34,6 +34,7 @@
 #include <sys/capability.h>
 #endif
 
+#include "config.h"
 #include "version.h"
 
 #define MAX(a, b)  (((a) > (b)) ? (a) : (b))
@@ -132,6 +133,7 @@ struct connection {
 struct listen_endpoint {
     int socketfd;       /* file descriptor of listening socket */
     int type;           /* SOCK_DGRAM | SOCK_STREAM */
+    int family;         /* AF_INET | AF_UNIX */
 };
 
 #define FD_CNXCLOSED    0
@@ -154,7 +156,7 @@ typedef enum {
 /* common.c */
 void init_cnx(struct connection *cnx);
 int set_nonblock(int fd);
-int connect_addr(struct connection *cnx, int fd_from, connect_blocking blocking);
+void connect_addr(struct connection *cnx, int fd_from, connect_blocking blocking);
 int fd2fd(struct queue *target, struct queue *from);
 char* sprintaddr(char* buf, size_t size, struct addrinfo *a);
 void resolve_name(struct addrinfo **out, char* fullname);
@@ -173,6 +175,7 @@ int resolve_split_name(struct addrinfo **out, char* hostname, char* port);
 int start_listen_sockets(struct listen_endpoint *sockfd[]);
 
 int defer_write(struct queue *q, void* data, ssize_t data_size);
+int defer_write_before(struct queue *q, void* data, ssize_t data_size);
 int flush_deferred(struct queue *q);
 
 extern struct sslhcfg_item cfg;
